package cn.ibizlab.businesscentral.core.sales.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.sales.domain.SalesOrder;
import cn.ibizlab.businesscentral.core.sales.filter.SalesOrderSearchContext;
import cn.ibizlab.businesscentral.core.sales.service.ISalesOrderService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.sales.mapper.SalesOrderMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[销售订单] 服务对象接口实现
 */
@Slf4j
@Service("SalesOrderServiceImpl")
public class SalesOrderServiceImpl extends ServiceImpl<SalesOrderMapper, SalesOrder> implements ISalesOrderService {

    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.finance.service.IInvoiceService invoiceService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.IOrderCloseService ordercloseService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.ISalesOrderDetailService salesorderdetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.marketing.service.ICampaignService campaignService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.IOpportunityService opportunityService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.product.service.IPriceLevelService pricelevelService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.sales.service.IQuoteService quoteService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ISlaService slaService;
    @Autowired
    @Lazy
    private cn.ibizlab.businesscentral.core.base.service.ITransactionCurrencyService transactioncurrencyService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(SalesOrder et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SalesOrder et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<SalesOrder> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<SalesOrder> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public SalesOrder getDraft(SalesOrder et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(SalesOrder et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("salesorderid",et.getSalesorderid())))
            return false;
        CachedBeanCopier.copy(get(et.getSalesorderid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SalesOrder> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(SalesOrder et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getSalesorderid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SalesOrder> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public SalesOrder get(String key) {
        SalesOrder et = getById(key);
        if(et==null){
            et=new SalesOrder();
            et.setSalesorderid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public boolean checkKey(SalesOrder et) {
        return (!ObjectUtils.isEmpty(et.getSalesorderid()))&&(!Objects.isNull(this.getById(et.getSalesorderid())));
    }


	@Override
    public List<SalesOrder> selectByCampaignid(String campaignid) {
        return baseMapper.selectByCampaignid(campaignid);
    }

    @Override
    public void removeByCampaignid(String campaignid) {
        this.remove(new QueryWrapper<SalesOrder>().eq("campaignid",campaignid));
    }

	@Override
    public List<SalesOrder> selectByOpportunityid(String opportunityid) {
        return baseMapper.selectByOpportunityid(opportunityid);
    }

    @Override
    public void removeByOpportunityid(String opportunityid) {
        this.remove(new QueryWrapper<SalesOrder>().eq("opportunityid",opportunityid));
    }

	@Override
    public List<SalesOrder> selectByPricelevelid(String pricelevelid) {
        return baseMapper.selectByPricelevelid(pricelevelid);
    }

    @Override
    public void removeByPricelevelid(String pricelevelid) {
        this.remove(new QueryWrapper<SalesOrder>().eq("pricelevelid",pricelevelid));
    }

	@Override
    public List<SalesOrder> selectByQuoteid(String quoteid) {
        return baseMapper.selectByQuoteid(quoteid);
    }

    @Override
    public void removeByQuoteid(String quoteid) {
        this.remove(new QueryWrapper<SalesOrder>().eq("quoteid",quoteid));
    }

	@Override
    public List<SalesOrder> selectBySlaid(String slaid) {
        return baseMapper.selectBySlaid(slaid);
    }

    @Override
    public void removeBySlaid(String slaid) {
        this.remove(new QueryWrapper<SalesOrder>().eq("slaid",slaid));
    }

	@Override
    public List<SalesOrder> selectByTransactioncurrencyid(String transactioncurrencyid) {
        return baseMapper.selectByTransactioncurrencyid(transactioncurrencyid);
    }

    @Override
    public void removeByTransactioncurrencyid(String transactioncurrencyid) {
        this.remove(new QueryWrapper<SalesOrder>().eq("transactioncurrencyid",transactioncurrencyid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SalesOrder> searchDefault(SalesOrderSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SalesOrder> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SalesOrder>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }





    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<SalesOrder> getSalesorderByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<SalesOrder> getSalesorderByEntities(List<SalesOrder> entities) {
        List ids =new ArrayList();
        for(SalesOrder entity : entities){
            Serializable id=entity.getSalesorderid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}


